@@ -558,8 +558,8 @@ class TestConnection(base.PyMySQLTestCase):
         sock.close()
 
     def test_ssl_connect(self):
-        dummy_ssl_context = mock.Mock(options=0)
-        with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+        dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+        with mock.patch(
             "pymysql.connections.ssl.create_default_context",
             new=mock.Mock(return_value=dummy_ssl_context),
         ) as create_default_context:
@@ -570,6 +570,7 @@ class TestConnection(base.PyMySQLTestCase):
                     "key": "key",
                     "cipher": "cipher",
                 },
+                defer_connect=True,
             )
             assert create_default_context.called
             assert dummy_ssl_context.check_hostname
@@ -581,8 +582,8 @@ class TestConnection(base.PyMySQLTestCase):
             )
             dummy_ssl_context.set_ciphers.assert_called_with("cipher")
 
-        dummy_ssl_context = mock.Mock(options=0)
-        with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+        dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+        with mock.patch(
             "pymysql.connections.ssl.create_default_context",
             new=mock.Mock(return_value=dummy_ssl_context),
         ) as create_default_context:
@@ -592,6 +593,7 @@ class TestConnection(base.PyMySQLTestCase):
                     "cert": "cert",
                     "key": "key",
                 },
+                defer_connect=True,
             )
             assert create_default_context.called
             assert dummy_ssl_context.check_hostname
@@ -603,8 +605,8 @@ class TestConnection(base.PyMySQLTestCase):
             )
             dummy_ssl_context.set_ciphers.assert_not_called
 
-        dummy_ssl_context = mock.Mock(options=0)
-        with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+        dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+        with mock.patch(
             "pymysql.connections.ssl.create_default_context",
             new=mock.Mock(return_value=dummy_ssl_context),
         ) as create_default_context:
@@ -615,6 +617,7 @@ class TestConnection(base.PyMySQLTestCase):
                     "key": "key",
                     "password": "password",
                 },
+                defer_connect=True,
             )
             assert create_default_context.called
             assert dummy_ssl_context.check_hostname
@@ -626,13 +629,14 @@ class TestConnection(base.PyMySQLTestCase):
             )
             dummy_ssl_context.set_ciphers.assert_not_called
 
-        dummy_ssl_context = mock.Mock(options=0)
-        with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+        dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+        with mock.patch(
             "pymysql.connections.ssl.create_default_context",
             new=mock.Mock(return_value=dummy_ssl_context),
         ) as create_default_context:
             pymysql.connect(
                 ssl_ca="ca",
+                defer_connect=True,
             )
             assert create_default_context.called
             assert not dummy_ssl_context.check_hostname
@@ -640,8 +644,8 @@ class TestConnection(base.PyMySQLTestCase):
             dummy_ssl_context.load_cert_chain.assert_not_called
             dummy_ssl_context.set_ciphers.assert_not_called
 
-        dummy_ssl_context = mock.Mock(options=0)
-        with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+        dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+        with mock.patch(
             "pymysql.connections.ssl.create_default_context",
             new=mock.Mock(return_value=dummy_ssl_context),
         ) as create_default_context:
@@ -649,6 +653,7 @@ class TestConnection(base.PyMySQLTestCase):
                 ssl_ca="ca",
                 ssl_cert="cert",
                 ssl_key="key",
+                defer_connect=True,
             )
             assert create_default_context.called
             assert not dummy_ssl_context.check_hostname
@@ -661,8 +666,8 @@ class TestConnection(base.PyMySQLTestCase):
             dummy_ssl_context.set_ciphers.assert_not_called
 
         for ssl_verify_cert in (True, "1", "yes", "true"):
-            dummy_ssl_context = mock.Mock(options=0)
-            with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+            dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+            with mock.patch(
                 "pymysql.connections.ssl.create_default_context",
                 new=mock.Mock(return_value=dummy_ssl_context),
             ) as create_default_context:
@@ -670,6 +675,7 @@ class TestConnection(base.PyMySQLTestCase):
                     ssl_cert="cert",
                     ssl_key="key",
                     ssl_verify_cert=ssl_verify_cert,
+                    defer_connect=True,
                 )
                 assert create_default_context.called
                 assert not dummy_ssl_context.check_hostname
@@ -682,8 +688,8 @@ class TestConnection(base.PyMySQLTestCase):
                 dummy_ssl_context.set_ciphers.assert_not_called
 
         for ssl_verify_cert in (None, False, "0", "no", "false"):
-            dummy_ssl_context = mock.Mock(options=0)
-            with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+            dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+            with mock.patch(
                 "pymysql.connections.ssl.create_default_context",
                 new=mock.Mock(return_value=dummy_ssl_context),
             ) as create_default_context:
@@ -691,6 +697,7 @@ class TestConnection(base.PyMySQLTestCase):
                     ssl_cert="cert",
                     ssl_key="key",
                     ssl_verify_cert=ssl_verify_cert,
+                    defer_connect=True,
                 )
                 assert create_default_context.called
                 assert not dummy_ssl_context.check_hostname
@@ -704,8 +711,8 @@ class TestConnection(base.PyMySQLTestCase):
 
         for ssl_ca in ("ca", None):
             for ssl_verify_cert in ("foo", "bar", ""):
-                dummy_ssl_context = mock.Mock(options=0)
-                with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+                dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+                with mock.patch(
                     "pymysql.connections.ssl.create_default_context",
                     new=mock.Mock(return_value=dummy_ssl_context),
                 ) as create_default_context:
@@ -714,6 +721,7 @@ class TestConnection(base.PyMySQLTestCase):
                         ssl_cert="cert",
                         ssl_key="key",
                         ssl_verify_cert=ssl_verify_cert,
+                        defer_connect=True,
                     )
                     assert create_default_context.called
                     assert not dummy_ssl_context.check_hostname
@@ -727,8 +735,8 @@ class TestConnection(base.PyMySQLTestCase):
                     )
                     dummy_ssl_context.set_ciphers.assert_not_called
 
-        dummy_ssl_context = mock.Mock(options=0)
-        with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+        dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+        with mock.patch(
             "pymysql.connections.ssl.create_default_context",
             new=mock.Mock(return_value=dummy_ssl_context),
         ) as create_default_context:
@@ -737,6 +745,7 @@ class TestConnection(base.PyMySQLTestCase):
                 ssl_cert="cert",
                 ssl_key="key",
                 ssl_verify_identity=True,
+                defer_connect=True,
             )
             assert create_default_context.called
             assert dummy_ssl_context.check_hostname
@@ -748,8 +757,8 @@ class TestConnection(base.PyMySQLTestCase):
             )
             dummy_ssl_context.set_ciphers.assert_not_called
 
-        dummy_ssl_context = mock.Mock(options=0)
-        with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+        dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+        with mock.patch(
             "pymysql.connections.ssl.create_default_context",
             new=mock.Mock(return_value=dummy_ssl_context),
         ) as create_default_context:
@@ -759,6 +768,7 @@ class TestConnection(base.PyMySQLTestCase):
                 ssl_key="key",
                 ssl_key_password="password",
                 ssl_verify_identity=True,
+                defer_connect=True,
             )
             assert create_default_context.called
             assert dummy_ssl_context.check_hostname
@@ -770,8 +780,8 @@ class TestConnection(base.PyMySQLTestCase):
             )
             dummy_ssl_context.set_ciphers.assert_not_called
 
-        dummy_ssl_context = mock.Mock(options=0)
-        with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+        dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+        with mock.patch(
             "pymysql.connections.ssl.create_default_context",
             new=mock.Mock(return_value=dummy_ssl_context),
         ) as create_default_context:
@@ -782,11 +792,12 @@ class TestConnection(base.PyMySQLTestCase):
                     "cert": "cert",
                     "key": "key",
                 },
+                defer_connect=True,
             )
             assert not create_default_context.called
 
-        dummy_ssl_context = mock.Mock(options=0)
-        with mock.patch("pymysql.connections.Connection.connect"), mock.patch(
+        dummy_ssl_context = mock.Mock(options=0, verify_flags=0)
+        with mock.patch(
             "pymysql.connections.ssl.create_default_context",
             new=mock.Mock(return_value=dummy_ssl_context),
         ) as create_default_context:
@@ -795,6 +806,7 @@ class TestConnection(base.PyMySQLTestCase):
                 ssl_ca="ca",
                 ssl_cert="cert",
                 ssl_key="key",
+                defer_connect=True,
             )
             assert not create_default_context.called
 
@@ -854,7 +866,7 @@ class TestEscape(base.PyMySQLTestCase):
 
         mapping = con.encoders.copy()
         mapping[Foo] = escape_foo
-        #self.assertEqual(con.escape({"foo": Foo()}, mapping), {"foo": "bar"})
+        # self.assertEqual(con.escape({"foo": Foo()}, mapping), {"foo": "bar"})
         with self.assertRaises(TypeError):
             con.escape({"foo": Foo()})
 
@@ -883,3 +895,40 @@ class TestEscape(base.PyMySQLTestCase):
         con.commit()
         cur.execute("SELECT 3")
         self.assertEqual(cur.fetchone()[0], 3)
+
+    def test_force_close_closes_socketio(self):
+        con = self.connect()
+        sock = con._sock
+        fileno = sock.fileno()
+        rfile = con._rfile
+
+        con._force_close()
+        assert rfile.closed
+        assert sock._closed
+        assert sock.fileno() != fileno  # should be set to -1
+
+    def test_socket_closed_on_exception_in_connect(self):
+        con = self.connect(defer_connect=True)
+        sock = None
+        rfile = None
+        fileno = -1
+
+        def _request_authentication():
+            nonlocal sock, rfile, fileno
+            sock = con._sock
+            assert sock is not None
+            fileno = sock.fileno()
+            rfile = con._rfile
+            assert rfile is not None
+            raise TypeError
+
+        con._request_authentication = _request_authentication
+
+        with pytest.raises(TypeError):
+            con.connect()
+        assert not con.open
+        assert con._rfile is None
+        assert con._sock is None
+        assert rfile.closed
+        assert sock._closed
+        assert sock.fileno() != fileno  # should be set to -1
